@@ -1,4 +1,4 @@
-from devoud.app import *
+from devoud.app import main
 
 if __name__ == '__main__':
     main()
