@@ -2,18 +2,17 @@ import json
 import os
 from platform import system
 
-from PySide6.QtCore import QSize, Slot
+from PySide6.QtCore import QSize, Slot, QTimer
 from PySide6.QtGui import QPixmap, QIcon, Qt
 from PySide6.QtWidgets import QHBoxLayout, QStackedWidget, QLabel, QPushButton, QComboBox, QSizePolicy, QSpacerItem, \
     QWidget, QVBoxLayout, QListWidget, QListWidgetItem, QScrollArea, QProgressBar, QCheckBox, QTreeWidgetItem, \
-    QMessageBox, QHeaderView
+    QMessageBox, QHeaderView, QFrame
 
 from devoud import __name__, __version__, __license__, __author__, IS_PYINSTALLER
 from devoud import rpath, restart
-from devoud.browser.downloads import DownloadItem
 from devoud.browser.page.embedded.view import EmbeddedView
 from devoud.browser.page.web import search_engines
-from devoud.browser.utils import human_bytes, open_in_file_manager
+from devoud.browser.utils import human_bytes, get_folder_size, open_in_file_manager
 from devoud.browser.utils.shortcuts import make_shortcut
 from devoud.browser.utils.versions import last_version
 from devoud.browser.widgets.container import ContainerWidget
@@ -65,17 +64,20 @@ class ControlPage(EmbeddedView):
                                                                f'\nРазработал {__author__}\nс '
                                                                f'использованием PySide6 под лицензией {__license__}'),
                                                    0, 1)
-        self.update_program_button = QPushButton(self.about_widget, text='Проверить обновления')
-        self.update_program_button.setFixedSize(165, 22)
-        self.update_program_button.setObjectName('container_title_button')
-        self.update_program_button.clicked.connect(self.update_program)
-        self.about_widget.title_layout.addWidget(self.update_program_button)
+        if system() != 'Windows' and not IS_PYINSTALLER:
+            # FIXME: Windows не дает перезаписывать используемые файлы
+            self.update_program_button = QPushButton(self.about_widget, text='Проверить обновления')
+            self.update_program_button.setFixedSize(165, 22)
+            self.update_program_button.setObjectName('container_title_button')
+            self.update_program_button.clicked.connect(self.update_program)
+            self.about_widget.title_layout.addWidget(self.update_program_button)
 
-        self.create_shortcut_button = QPushButton(self.about_widget, text='Создать ярлык')
-        self.create_shortcut_button.setFixedSize(110, 22)
-        self.create_shortcut_button.setObjectName('container_title_button')
-        self.create_shortcut_button.clicked.connect(self.create_shortcut)
-        self.about_widget.title_layout.addWidget(self.create_shortcut_button)
+        if not IS_PYINSTALLER:
+            self.create_shortcut_button = QPushButton(self.about_widget, text='Создать ярлык')
+            self.create_shortcut_button.setFixedSize(110, 22)
+            self.create_shortcut_button.setObjectName('container_title_button')
+            self.create_shortcut_button.clicked.connect(self.create_shortcut)
+            self.about_widget.title_layout.addWidget(self.create_shortcut_button)
 
         self.project_site_button = QPushButton(self.about_widget, text='Страница проекта')
         self.project_site_button.setFixedSize(130, 22)
@@ -103,6 +105,7 @@ class ControlPage(EmbeddedView):
         self.restart_button.clicked.connect(self.window.restart)
         self.settings_widget.title_layout.addWidget(self.restart_button)
 
+        # Чекбоксы
         self.history_checkbox = self.SettingsCheckBox(parent=self,
                                                       text='Сохранять историю',
                                                       option='saveHistory',
@@ -147,42 +150,68 @@ class ControlPage(EmbeddedView):
                                                             restart=True)
         self.settings_widget.content_layout.addWidget(self.smooth_scroll_checkbox, 6, 0)
 
+        self.home_frame = QFrame(self)
+        self.home_frame.setLayout(QHBoxLayout())
+        self.home_frame.layout().setContentsMargins(0, 0, 0, 0)
         self.home_lineedit = LineEdit(self)
         self.home_lineedit.setFixedSize(QSize(215, 24))
         self.home_lineedit.setFocusPolicy(Qt.ClickFocus)
         self.home_lineedit.textEdited.connect(lambda: self.settings.set('homePage', self.home_lineedit.text()))
         self.home_lineedit.setText(self.settings.get('homePage'))
-        self.settings_widget.content_layout.addWidget(self.home_lineedit, 7, 0)
-        self.settings_widget.content_layout.addWidget(QLabel(self.settings_widget, text='Домашняя страница'), 7, 1)
+        self.home_frame.layout().addWidget(self.home_lineedit)
+        self.home_frame.layout().addWidget(QLabel(self.settings_widget, text='Домашняя страница'))
+        self.settings_widget.content_layout.addWidget(self.home_frame, 7, 0)
 
+        self.new_page_frame = QFrame(self)
+        self.new_page_frame.setLayout(QHBoxLayout())
         self.new_page_box = QComboBox(self.settings_widget)
+        self.new_page_frame.layout().setContentsMargins(0, 0, 0, 0)
         [self.new_page_box.addItem(title) for title in ['Заставка с часами', 'Поисковик', 'Домашняя страница']]
         self.new_page_box.setFixedSize(QSize(215, 24))
         self.new_page_box.setFocusPolicy(Qt.ClickFocus)
         self.new_page_box.setCurrentText(self.settings.get('newPage')['title'])
         self.new_page_box.currentIndexChanged.connect(self.change_new_page)
-        self.settings_widget.content_layout.addWidget(self.new_page_box, 8, 0)
-        self.settings_widget.content_layout.addWidget(QLabel("Новая страница"), 8, 1)
+        self.new_page_frame.layout().addWidget(self.new_page_box)
+        self.new_page_frame.layout().addWidget(QLabel("Новая страница"))
+        self.settings_widget.content_layout.addWidget(self.new_page_frame, 8, 0)
 
+        self.search_frame = QFrame(self)
+        self.search_frame.setLayout(QHBoxLayout())
+        self.search_frame.layout().setContentsMargins(0, 0, 0, 0)
         self.search_box = QComboBox(self.settings_widget)
         [self.search_box.addItem(QIcon(search_engines[title]['icon']), title) for title in search_engines.keys()]
         self.search_box.setFixedSize(QSize(215, 24))
         self.search_box.setFocusPolicy(Qt.ClickFocus)
         self.search_box.setCurrentText(self.settings.get('searchEngine'))
         self.search_box.currentIndexChanged.connect(self.change_default_search)
-        self.settings_widget.content_layout.addWidget(self.search_box, 9, 0)
-        self.settings_widget.content_layout.addWidget(QLabel('Поисковая система по умолчанию'), 9, 1)
+        self.search_frame.layout().addWidget(self.search_box)
+        self.search_frame.layout().addWidget(QLabel('Поисковая система по умолчанию'))
+        self.settings_widget.content_layout.addWidget(self.search_frame, 9, 0)
 
+        self.tab_bar_position_frame = QFrame(self)
+        self.tab_bar_position_frame.setLayout(QHBoxLayout())
+        self.tab_bar_position_frame.layout().setContentsMargins(0, 0, 0, 0)
         self.tab_bar_position_box = QComboBox(self.settings_widget)
         self.tab_bar_position_box.addItem('Снизу')
         self.tab_bar_position_box.addItem('Сверху')
         self.tab_bar_position_box.setCurrentText(self.settings.get('TabBarPosition'))
         self.tab_bar_position_box.setFixedSize(QSize(215, 24))
         self.tab_bar_position_box.setFocusPolicy(Qt.ClickFocus)
-        self.settings_widget.content_layout.addWidget(self.tab_bar_position_box, 10, 0)
+        self.tab_bar_position_frame.layout().addWidget(self.tab_bar_position_box)
+        self.tab_bar_position_frame.layout().addWidget(QLabel("Положение панели вкладок"))
+        self.settings_widget.content_layout.addWidget(self.tab_bar_position_frame, 10, 0)
         self.tab_bar_position_box.currentIndexChanged.connect(
             lambda: self.change_tab_bar_position(self.tab_bar_position_box.currentText()))
-        self.settings_widget.content_layout.addWidget(QLabel("Положение панели вкладок"), 10, 1)
+
+        self.cache_frame = QFrame(self)
+        self.cache_frame.setLayout(QHBoxLayout())
+        self.cache_label = QLabel(f'Кеш: {human_bytes(get_folder_size(f"{self.FS.user_dir()}/cache"))}')
+        self.clear_cache_button = QPushButton('Очистить')
+        self.clear_cache_button.clicked.connect(self.clear_cache)
+        self.clear_cache_button.setFixedWidth(100)
+        self.cache_frame.layout().addWidget(self.cache_label)
+        self.cache_frame.layout().addWidget(self.clear_cache_button)
+        self.settings_widget.content_layout.addWidget(self.cache_frame, 11, 0, alignment=Qt.AlignLeft)
 
         # Виджет тем
         self.themes_widget = ContainerWidget(self, 'Темы')
@@ -203,6 +232,7 @@ class ControlPage(EmbeddedView):
         self.history_widget.content_layout.addWidget(self.history_list_widget)
         self.window.history.history_add.connect(self.history_list_widget.add)
         self.window.history.history_remove.connect(self.history_list_widget.remove)
+        self.window.history.history_remove_all.connect(self.history_list_widget.clear)
         for item in self.window.history.get_all():
             self.history_list_widget.add(item)
 
@@ -214,6 +244,7 @@ class ControlPage(EmbeddedView):
         self.bookmarks_section.add_widget(self.bookmarks_widget)
         self.window.bookmarks.bookmark_add.connect(self.bookmarks_list_widget.add)
         self.window.bookmarks.bookmark_remove.connect(self.bookmarks_list_widget.remove)
+        self.window.bookmarks.bookmark_remove_all.connect(self.bookmarks_list_widget.clear)
         for item in self.window.bookmarks.get_all():
             self.bookmarks_list_widget.add(item)
 
@@ -224,22 +255,24 @@ class ControlPage(EmbeddedView):
         self.download_widget.content_layout.addWidget(self.download_list_widget)
         self.download_section.add_widget(self.download_widget)
 
-        downloads = self.window.download_manager.download_queue()
-        download_history = list(self.window.download_manager.history().values())
-        for item in downloads + download_history:
+        for item in self.window.downloads.get_all():
             self.download_list_widget.add(item)
-        self.window.download_manager.download_add.connect(self.download_list_widget.add)
-        self.window.download_manager.download_remove.connect(self.download_list_widget.remove)
+        self.window.downloads.download_add.connect(self.download_list_widget.add)
+        self.window.downloads.download_remove.connect(self.download_list_widget.remove)
+        self.window.downloads.download_remove_all.connect(self.download_list_widget.clear)
 
         self.sections_panel.setCurrentRow(self.section_tags.get(self.abstract_page.url, 0))
 
     def deleteLater(self) -> None:
         self.window.bookmarks.bookmark_add.disconnect(self.bookmarks_list_widget.add)
         self.window.bookmarks.bookmark_remove.disconnect(self.bookmarks_list_widget.remove)
-        self.window.download_manager.download_add.disconnect(self.download_list_widget.add)
-        self.window.download_manager.download_remove.disconnect(self.download_list_widget.remove)
+        self.window.bookmarks.bookmark_remove_all.disconnect(self.bookmarks_list_widget.clear)
+        self.window.downloads.download_add.disconnect(self.download_list_widget.add)
+        self.window.downloads.download_remove.disconnect(self.download_list_widget.remove)
+        self.window.downloads.download_remove_all.disconnect(self.download_list_widget.clear)
         self.window.history.history_add.disconnect(self.history_list_widget.add)
         self.window.history.history_remove.disconnect(self.history_list_widget.remove)
+        self.window.history.history_remove_all.disconnect(self.history_list_widget.clear)
         super().deleteLater()
 
     @Slot(int)
@@ -302,6 +335,12 @@ class ControlPage(EmbeddedView):
         self.settings.set('TabBarPosition', position)
         self.window.tab_widget.set_tab_bar_position(position)
 
+    def clear_cache(self):
+        if QMessageBox.question(self.abstract_page, 'Подтверждение операции',
+                                'Это действие удалит весь кеш программы. Продолжить?') == QMessageBox.Yes:
+            self.FS.clear_cache()
+            QTimer.singleShot(100, lambda: self.cache_label.setText(f'Кеш: {human_bytes(get_folder_size(f"{self.FS.user_dir()}/cache"))}'))
+
     @Slot()
     def update_program(self):
         msg = QMessageBox(self.window)
@@ -438,28 +477,30 @@ class ControlPage(EmbeddedView):
 
             self.empty_info_label.setText('Список загрузок пуст')
 
-        @Slot(DownloadItem)
-        def add(self, item: DownloadItem):
-            if self.findItems(item.name, Qt.MatchContains, 0):
-                return False
-            download_item = QTreeWidgetItem([item.name, None, None])
-            download_item.d_item = item
-            self.insertTopLevelItem(0, download_item)
+        @Slot(object)
+        def add(self, item):
+            old_item = self.findItems(item.name, Qt.MatchContains, 0)
+            if old_item:
+                print('REMOVE OLD DOWNLOAD', old_item[0])
+                self.takeTopLevelItem(self.indexOfTopLevelItem(old_item[0]))
+            widget_item = QTreeWidgetItem([item.name, None, None])
+            widget_item.download = item
+            self.insertTopLevelItem(0, widget_item)
             self.count += 1
             download_item_size = QTreeWidgetItem()
             download_item_size.setText(0, 'Размер')
             download_item_size.setText(2, human_bytes(item.total_bytes))
-            download_item.addChild(download_item_size)
+            widget_item.addChild(download_item_size)
 
             download_item_date = QTreeWidgetItem()
             download_item_date.setText(0, 'Дата')
-            download_item_date.setText(2, item.date)
-            download_item.addChild(download_item_date)
+            download_item_date.setText(2, item.date.strftime("%d-%m-%Y %H:%M"))
+            widget_item.addChild(download_item_date)
 
             download_item_source = QTreeWidgetItem()
             download_item_source.setText(0, 'Источник')
             download_item_source.setText(2, item.url)
-            download_item.addChild(download_item_source)
+            widget_item.addChild(download_item_source)
 
             download_item_buttons = QWidget()
             download_item_buttons.setLayout(QHBoxLayout())
@@ -472,20 +513,21 @@ class ControlPage(EmbeddedView):
             folder_button.setFixedSize(30, 30)
             download_item_buttons.layout().addWidget(folder_button)
 
-            self.setItemWidget(download_item, 1, download_item_buttons)
+            self.setItemWidget(widget_item, 1, download_item_buttons)
 
             item_progress_bar = QProgressBar()
-            self.setItemWidget(download_item, 2, item_progress_bar)
+            self.setItemWidget(widget_item, 2, item_progress_bar)
 
-            if item.request:
+            if hasattr(item, 'request'):
                 item.request.isFinishedChanged.connect(
                     lambda: self._download_finished(item, item_progress_bar, download_item_size))
-                item.request.receivedBytesChanged.connect(lambda: item_progress_bar.setValue(item.percent))
+                item.request.receivedBytesChanged.connect(
+                    lambda: self._received_bytes_changed(item, item_progress_bar, download_item_size)),
                 item.request.totalBytesChanged.connect(
                     lambda: download_item_size.setText(2, human_bytes(item.total_bytes)))
             else:
-                item_progress_bar.setValue(100)
-                item_progress_bar.setFormat('Завершен')
+                item_progress_bar.setValue(int(item.received_bytes * 100 / item.total_bytes))
+                item_progress_bar.setFormat(item.status)
             self.show_empty_message()
 
         def context_menu(self, event):
@@ -496,18 +538,36 @@ class ControlPage(EmbeddedView):
                 return
 
             item = self.itemAt(event)
-            deleted_text = 'Отменить загрузку' if item.d_item.request else 'Убрать'
-            menu.addAction(deleted_text, item.d_item.remove)
+            if not item.parent():
+                if hasattr(item.download, 'request'):
+                    deleted_text = 'Отменить загрузку'
+                    remove_action = item.download.request.cancel
+                else:
+                    deleted_text = 'Убрать'
+                    remove_action = lambda: self.window().downloads.remove(item.download)
+                menu.addAction(deleted_text, remove_action)
             menu.popup(self.mapToGlobal(event))
 
-        @staticmethod
-        def _download_finished(item: DownloadItem, progress_bar, size_text):
-            progress_bar.setValue(100)
-            progress_bar.setFormat('Завершен')
-            size_text.setText(2, human_bytes(item.total_bytes))
+        @Slot()
+        def _received_bytes_changed(self, item, progress_bar, size_label):
+            try:
+                progress_bar.setValue(int(item.request.receivedBytes() * 100 / item.total_bytes))
+            except OverflowError:
+                pass
+            size_label.setText(2, human_bytes(item.total_bytes))
 
-        @Slot(DownloadItem)
-        def remove(self, item: DownloadItem):
+        @Slot()
+        def _download_finished(self, item, progress_bar, size_label):
+            status_text = self.window().downloads.state_dict.get(item.request.state(), {
+                'notify_title': 'Файл не был загружен',
+                'status': 'Неизвестно'
+            })['status']
+            progress_bar.setValue(int(item.request.receivedBytes() * 100 / item.total_bytes))
+            size_label.setText(2, human_bytes(item.total_bytes))
+            progress_bar.setFormat(status_text)
+
+        @Slot(object)
+        def remove(self, item):
             items = self.findItems(item.name, Qt.MatchContains, 0)
             for i in items:
                 self.takeTopLevelItem(self.indexOfTopLevelItem(i))
@@ -523,7 +583,7 @@ class ControlPage(EmbeddedView):
             msg.setInformativeText('Загружаемые файлы будут отменены')
             msg.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
             if msg.exec() == QMessageBox.Yes:
-                self.window().download_manager.remove_all()
+                self.window().downloads.remove_all()
 
     class BookmarksListWidget(TreeWidget):
         def __init__(self, parent):
