@@ -543,9 +543,12 @@ class Theme:
             color: $find_close_fg;
             border: 0;
         }
+        
+        #title_bar_icon {
+            margin-top: 7px;
+        }
 
         #title_bar_label {
-            margin-left: 12px;
             margin-top: 5px;
         }
 
