@@ -1,5 +1,6 @@
 import os
 import json
+import sys
 
 from pkg_resources import parse_version
 from urllib import request
@@ -28,6 +29,8 @@ def cli_update(package_name: str):
         print('[Обновление]: У вас установлена последняя версия программы')
         return
     print(f'[Обновление]: Доступно {__version__} -> {last_version(__name__)}')
+    if sys.platform == 'win32':
+        return print('[Обновление]: Данная опция не доступна для систем Windows, используйте "pip install devoud --upgrade"')
     if input('[Обновление]: Установить обновления? [y/n]: ').lower() == 'y':
         os.system(f'pip3 install {package_name} --upgrade')
     else:
